// Arduino Field Server Shield用プログラム
// 2014.7.11 ver.1.0  T.Fukatsu
//
#include <SPI.h>
#include <Ethernet.h>

// HTML出力用雛型
PROGMEM prog_char version_name[] = "[FS-classic simple v1.0]";
PROGMEM prog_char web_header[] = "HTTP/1.0 200 OK\nContent-Type: text/html\nConnection: close\n";
PROGMEM prog_char web_top1[]   = "<html><head><title>Field Server Classic Web Server</title></";
PROGMEM prog_char web_top2[]   = "head><body><h3>Field Server Classic ver.1.18</h3><hr/><form>";
PROGMEM prog_char web_top3[]   = "<table border>";
PROGMEM prog_char web_end[] = "</table></form><HR/>2013.9.4 by T.FUKATSU(NARO)</body></html>";

// Arduinoのpin設定
const byte DC5V_pin = 2;       //イーサシールド&無線用電源(CN10:5V/CN9:12V)をon/offするリレー
const byte DC12V_pin1 = 3;     //電動FANの電源(CN7:12V)をon/offするリレー
const byte DC12V_pin2 = 4;     //外部カメラの電源(CN8:12V)をon/offするリレー
const byte reset_pin = 5;      //イーサシールドのリセットピン(起動時にHにする必要有)

// インターネット接続関係
EthernetServer server(80);
byte mac[] = { 0x90, 0xA2, 0xDA, 0x0D, 0xA4, 0xA4 }; //イーサシールドの裏を参照
byte  ip[] = { 192, 168, 10, 200 }; //ArduinoのIPアドレス

//データ取得時の変数
unsigned int raw[6]; // A/Dで得られた生電圧を保存(0-1023)
char str[64]; // コメント表示,Twitterなどテンポラリ文字列を格納
char msg[141]; // Twitteする本文を格納,最大140文字


//
// セットアップ
//
void setup(){ 

  // シリアル表示の設定
  Serial.begin(9600);  
  strcpy_P(str,(char*)version_name);
  Serial.println(str);

  // pin setup
  pinMode (DC5V_pin, OUTPUT);
  pinMode (DC12V_pin1, OUTPUT);
  pinMode (DC12V_pin2, OUTPUT);
  pinMode (reset_pin, OUTPUT);

  // power on
  digitalWrite(reset_pin, LOW); //最初はLOWに設定
  digitalWrite(DC5V_pin, HIGH); //イーサシールド&無線LANの電源ON
  digitalWrite(DC12V_pin1, HIGH); //電動ファンの電源ON
  digitalWrite(DC12V_pin2, HIGH); //カメラの電源ON
  digitalWrite(reset_pin, HIGH); //Etherシールドをリセットして起動

  // Ethernet setup
  Ethernet.begin(mac, ip);
  server.begin();

  // A/D読み取り範囲設定(何もしない場合は0-5V)
  //analogReference(INTERNAL); //アナログ入力を0-1Vに切り替え  
}


//
// メインループ
//
void loop(){

  // Web serverの表示
  EthernetClient client = server.available();
  String arg_data = ""; //Webサーバアクセス時の引数を初期化
  boolean arg_flag = true; //HTTPのヘッダを読み込み終えるまでtrue

  if (client) {
    // an http request ends with a blank line
    boolean currentLineIsBlank = true;
    while (client.connected()) {
      delay(1); //Webサーバがハングしないようウェイト
      if (client.available()) {
        char c = client.read();
        if( arg_flag ) arg_data.concat(c);
        if (c == '\n' && currentLineIsBlank) {

          //引数を確認して動作
          //カメラの電源制御
          if(arg_data.indexOf("relay2=off") != -1 ){
            digitalWrite(DC12V_pin2,LOW);
          }
          if(arg_data.indexOf("relay2=on") != -1 ){
            digitalWrite(DC12V_pin2,HIGH);
          }
          //ファンの電源制御               
          if(arg_data.indexOf("relay1=off") != -1 ){
            digitalWrite(DC12V_pin1,LOW);
          }
          if(arg_data.indexOf("relay1=on") != -1 ){
            digitalWrite(DC12V_pin1,HIGH);
          }  
          
          // Webレスポンス
          strcpy_P(str, (char*)web_header);
          client.println(str);                
          strcpy_P(str, (char*)web_top1);
          client.print(str);
          strcpy_P(str, (char*)web_top2);
          client.print(str);
          strcpy_P(str, (char*)web_top3);
          client.println(str);
          client.print(F("<tr><td>"));                
          strcpy_P(str, (char*)version_name);
          client.print(str);
          client.println(F(" </td></tr>"));               
          ReadSensor();             
          for (int analogChannel = 0; analogChannel < 6; analogChannel++) {
            client.print(F("<tr><td>ad"));
            client.print(analogChannel);
            client.print(F(":"));
            client.print(raw[analogChannel]);
            client.println(F("<br /></td></tr>"));
          }
          client.print(F("<tr><td>&nbsp;</td></tr>"));

          //カメラ(relay2)操作ボタン
          client.print(F("<tr><td>cam: <input type=\"submit\" name=\"relay2\" value=\"on\">"));
          client.print(F("<input type=\"submit\" name=\"relay2\" value=\"off\"></td></tr>"));
          //FAN(relay1)操作ボタン
          client.print(F("<tr><td>fan: <input type=\"submit\" name=\"relay1\" value=\"on\">"));
          client.print(F("<input type=\"submit\" name=\"relay1\" value=\"off\"></td></tr>"));
          //リロードボタン
          client.print(F("<tr><td><p align=\"center\"><input type=\"submit\" name=\"\" value=\"reload\"></p></td></tr>"));
          // 残り                
          strcpy_P(str, (char*)web_end);
          client.println(str);
          Serial.println(F("http OK."));
          break;
        }
        
        if (c == '\n') {
          // you're starting a new line
          arg_flag = false;
          currentLineIsBlank = true;
        } 
        else if (c != '\r') {
          // you've gotten a character on the current line
          currentLineIsBlank = false;
        }
      }
    }
    // give the web browser time to receive the data
    delay(1);
    // close the connection:
    client.stop();
  }
}


//
// A/D接続のセンサ値を読み込む(8回取りこんで平均する)
//
void ReadSensor(){
  for(byte analogChannel=0; analogChannel<6; analogChannel++) {
    analogRead(analogChannel); // dummy,チャンネル間の電圧吸い込み誤差を低減
    raw[analogChannel] = 0;
    for( byte i=0; i<8; i++){
      raw[analogChannel] += analogRead(analogChannel);
    }
    raw[analogChannel] = (raw[analogChannel]+4) >> 3; //切捨なので8/2を足してから8で割る
  }
}





